#!/usr/bin/env python
# encoding: utf-8

"""
    pytime
    ~~~~~~~~~~~~~

    A easy-use module to solve the datetime needs by string.

    :copyright: (c) 2015 by Sinux <nsinux@gmail.com>
    :license: MIT, see LICENSE for more details.
"""

import datetime
import calendar
from .filter import BaseParser


bp = BaseParser.main
dp = BaseParser.parse_diff


def parse(value):
    return bp(value)


def count(value1, value2):
    _val1, _val2 = parse(value1), parse(value2)
    if type(_val1) == type(_val2):
        return _val1 - _val2
    else:
        _val1 = _val1 if isinstance(_val1, datetime.datetime) else midnight(_val1)
        _val2 = _val2 if isinstance(_val2, datetime.datetime) else midnight(_val2)
        return _val1 - _val2


# max, min


_date = datetime.date.today()
_datetime = datetime.datetime.now()
_year = _date.year
_month = _date.month
_day = _date.day

_SEVEN_DAYS = datetime.timedelta(days=7)
_ONE_DAY = datetime.timedelta(days=1)


def today(year=None):
    """this day, last year"""
    return datetime.date(int(year), _date.month, _date.day) if year else _date


def tomorrow(date=None):
    """tomorrow is another day"""
    if not date:
        return _date + datetime.timedelta(days=1)
    else:
        current_date = parse(date)
        return current_date + datetime.timedelta(days=1)


def yesterday(date=None):
    """yesterday once more"""
    if not date:
        return _date - datetime.timedelta(days=1)
    else:
        current_date = parse(date)
        return current_date - datetime.timedelta(days=1)


########################
# function method
########################


def days_range(first=None, second=None, wipe=False):
    """
    get all days between first and second

    :param first: datetime, date or string
    :param second: datetime, date or string
    :param wipe: boolean, excludes first and last date from range when True. Default is False.
    :return: list
    """
    _first, _second = parse(first), parse(second)
    (_start, _end) = (_second, _first) if _first > _second else (_first, _second)
    days_between = (_end - _start).days
    date_list = [_end - datetime.timedelta(days=x) for x in range(0, days_between + 1)]
    if wipe and len(date_list) >= 2:
        date_list = date_list[1:-1]
    return date_list


def last_day(year=_year, month=_month):
    """
    get the current month's last day
    :param year:  default to current year
    :param month:  default to current month
    :return: month's last day
    """
    last_day = calendar.monthrange(year, month)[1]
    return datetime.date(year=year, month=month, day=last_day)


def midnight(arg=None):
    """
    convert date to datetime as midnight or get current day's midnight
    :param arg: string or date/datetime
    :return: datetime at 00:00:00
    """
    if arg:
        _arg = parse(arg)
        if isinstance(_arg, datetime.date):
            return datetime.datetime.combine(_arg, datetime.datetime.min.time())
        elif isinstance(_arg, datetime.datetime):
            return datetime.datetime.combine(_arg.date(), datetime.datetime.min.time())
    else:
        return datetime.datetime.combine(_date, datetime.datetime.min.time())


def before(base=_datetime, diff=None):
    """
    count datetime before `base` time
    :param base:  minuend -> str/datetime/date
    :param diff:  str
    :return: datetime
    """
    _base = parse(base)
    if isinstance(_base, datetime.date):
        _base = midnight(_base)
    if not diff:
        return _base
    result_dict = dp(diff)
    # weeks already convert to days in diff_parse function(dp)
    for unit in result_dict:
        _val = result_dict[unit]
        if not _val:
            continue
        if unit == 'years':
            _base = _base.replace(year=(_base.year - _val))
        elif unit == 'months':
            if _base.month <= _val:
                _month_diff = 12 - (_val - _base.month)
                _base = _base.replace(year=_base.year - 1).replace(month=_month_diff)
            else:
                _base = _base.replace(month=_base.month - _val)
        elif unit in ['days', 'hours', 'minutes', 'seconds']:
            _base = _base - datetime.timedelta(**{unit: _val})
    return _base


def after(base=_datetime, diff=None):
    """
    count datetime after diff args
    :param base: str/datetime/date
    :param diff: str
    :return: datetime
    """
    _base = parse(base)
    if isinstance(_base, datetime.date):
        _base = midnight(_base)
    result_dict = dp(diff)
    for unit in result_dict:
        _val = result_dict[unit]
        if not _val:
            continue
        if unit == 'years':
            _base = _base.replace(year=(_base.year + _val))
        elif unit == 'months':
            if _base.month + _val <= 12:
                _base = _base.replace(month=_base.month + _val)
            else:
                _month_diff = (_base.month + _val) - 12
                _base = _base.replace(year=_base.year + 1).replace(month=_month_diff)
        elif unit in ['days', 'hours', 'minutes', 'seconds']:
            _base = _base + datetime.timedelta(**{unit: _val})
    return _base


def _datetime_to_date(arg):
    """
    convert datetime/str to date
    :param arg:
    :return:
    """
    _arg = parse(arg)
    if isinstance(_arg, datetime.datetime):
        _arg = _arg.date()
    return _arg


# Monday to Monday -> 00:00:00 to 00:00:00   month 1st -  next month 1st
def this_week(arg=_date, clean=False):
    _arg = _datetime_to_date(arg)
    return _arg - datetime.timedelta(days=_arg.weekday()), _arg + datetime.timedelta(
        days=6 - _arg.weekday()) if clean else _arg + datetime.timedelta(days=6 - _arg.weekday()) + _ONE_DAY


def last_week(arg=_date, clean=False):
    this_week_tuple = this_week(arg)
    return this_week_tuple[0] - _SEVEN_DAYS, this_week_tuple[1] - _SEVEN_DAYS if clean \
        else this_week_tuple[1] - _SEVEN_DAYS + _ONE_DAY


def next_week(arg=_date, clean=False):
    this_week_tuple = this_week(arg)
    return this_week_tuple[0] + _SEVEN_DAYS, this_week_tuple[1] + _SEVEN_DAYS if clean \
        else this_week_tuple[1] + _SEVEN_DAYS + _ONE_DAY


def this_month(arg=_date, clean=False):
    _arg = _datetime_to_date(arg)
    return datetime.date(_arg.year, _arg.month, 1), last_day(_arg.year, _arg.month) if clean \
        else last_day(_arg.year, _arg.month) + _ONE_DAY


def last_month(arg=_date, clean=False):
    _arg = _datetime_to_date(arg)
    this_month_first_day = datetime.date(_arg.year, _arg.month, 1)
    last_month_last_day = this_month_first_day - _ONE_DAY
    last_month_first_day = datetime.date(last_month_last_day.year, last_month_last_day.month, 1)
    return last_month_first_day, last_month_last_day if clean else this_month_first_day


def next_month(arg=_date, clean=False):
    _arg = _datetime_to_date(arg)
    this_month_last_day = last_day(_arg.year, _arg.month)
    next_month_first_day = this_month_last_day + _ONE_DAY
    next_month_last_day = last_day(next_month_first_day.year, next_month_first_day.month)
    return next_month_first_day, next_month_last_day if clean else next_month_last_day + _ONE_DAY


######################
# festival
######################


def new_year(year=None):
    return datetime.date(int(year), 1, 1) if year else datetime.date(_year, 1, 1)


def valentine(year=None):
    return datetime.date(int(year), 2, 14) if year else datetime.date(_year, 2, 14)


def fool(year=None):
    return datetime.date(int(year), 4, 1) if year else datetime.date(_year, 4, 1)


def christmas(year=None):
    return datetime.date(int(year), 12, 25) if year else datetime.date(_year, 12, 25)


def christ_eve(year=None):
    return yesterday(christmas(year))


def mother(year=None):
    """
    the 2nd Sunday in May
    :param year: int
    :return: Mother's day
    """
    may_first = datetime.date(_year, 5, 1) if not year else datetime.date(int(year), 5, 1)
    weekday_seq = may_first.weekday()
    return datetime.date(may_first.year, 5, (14 - weekday_seq))


def father(year=None):
    """
    the 3rd Sunday in June
    :param year: int
    :return: Father's day
    """
    june_first = datetime.date(_year, 6, 1) if not year else datetime.date(int(year), 6, 1)
    weekday_seq = june_first.weekday()
    return datetime.date(june_first.year, 6, (21 - weekday_seq))


def halloween(year=None):
    return last_day(month=10) if not year else last_day(year, 10)


def easter(year=None):
    """
    1900 - 2099 limit
    :param year: int
    :return: Easter day
    """
    y = int(year) if year else _year
    n = y - 1900
    a = n % 19
    q = n // 4
    b = (7 * a + 1) // 19
    m = (11 * a + 4 - b) % 29
    w = (n + q + 31 - m) % 7
    d = 25 - m - w
    if d > 0:
        return datetime.date(y, 4, d)
    else:
        return datetime.date(y, 3, (31 + d))


def thanks(year=None):
    """
    4rd Thursday in Nov
    :param year: int
    :return: Thanksgiving Day
    """
    nov_first = datetime.date(_year, 11, 1) if not year else datetime.date(int(year), 11, 1)
    weekday_seq = nov_first.weekday()
    if weekday_seq > 3:
        current_day = 32 - weekday_seq
    else:
        current_day = 25 - weekday_seq
    return datetime.date(nov_first.year, 11, current_day)


def vatertag(year=None):
    """
    father's day in Germany
    """
    return after(easter(year), '39days').date()


if __name__ == '__main__':
    # _time_filter('2015-01-03')
    # print(calendar.monthrange(2015, 10))
    print(bp('2015-01-03'))



#!/usr/bin/env python
# encoding: utf-8

import sys
import datetime
import re
from itertools import chain
from .exception import UnexpectedTypeError, CanNotFormatError


py = sys.version_info
py3k = py >= (3, 0, 0)
py2k = (2, 6, 0) < py < (3, 0, 0)
py25 = py < (2, 6, 0)

str_tuple = (str,) if py3k else (str, unicode)

_current = str(datetime.datetime.today().date())


def _exchange_y_d(string, y_l):
    d_l = 2 if len(string.split('-')[2]) > 1 else 1
    _st_l = list(string)
    _st_l[:d_l], _st_l[-y_l:] = _st_l[-y_l:], _st_l[:d_l]
    return ''.join(_st_l)


# with letter `M` for minutes, `m` for month
UNIT_DICT = {'years': ['years', 'year', 'yea', 'ye', 'yr', 'y', 'Y'],
             'months': ['months', 'month', 'mont', 'mon', 'mo', 'm'],
             'weeks': ['weeks', 'week', 'wee', 'we', 'wk', 'w', 'W'],
             'days': ['days', 'day', 'dy', 'da', 'd', 'D'],
             'hours': ['hours', 'hour', 'hou', 'hr', 'ho', 'h', 'H'],
             'minutes': ['minutes', 'minute', 'minut', 'minu', 'min', 'mi', 'M'],
             'seconds': ['seconds', 'second', 'sec', 'se', 's', 'S']}

NAMED_MONTHS = {'Jan': 1, 'Feb': 2, 'Mar': 3, 'Apr': 4,
                'May': 5, 'Jun': 6, 'Jul': 7, 'Agu': 8,
                'Sep': 9, 'Oct': 10, 'Nov': 11, 'Dec': 12}


def filter_unit(arg):
    if len(arg) > 1:
        arg = arg.lower()
    result = [key for key in UNIT_DICT.keys() if arg in UNIT_DICT[key]]
    if result:
        return result[0]
    else:
        raise CanNotFormatError


class BaseParser(object):
    """Parse string to regular datetime/date type

    1990-10-28 23:23:23  - 19
    90-10-28 23:23:23    - 17

    1990-10-28           - 10
    28-10-1990           - 10
    1990/10/28           - 10
    28/10/1990           - 10
    1990.10.28           - 10
    28.10.1990           - 10

    10-28-90  USA        - 8
    28-10-90  on hold    - 8
    10/28/90  USA        - 8
    28/10/90  on hold    - 8

    19901028             - 8
    90-10-28             - 8
    90/10/28             - 8

    23:23:23             - 8

    23:23                - 5
    23:2                 - 4
    5:14                 - 4
    5:2                  - 3
    """

    def __init__(self, *args, **kwargs):
        pass

    @staticmethod
    def _str_parser(string):
        """
        return method by the length of string
        :param string: string
        :return: method
        """
        if not any(c.isalpha() for c in string):
            _string = string[:19]
            _length = len(_string)
            if _length > 10:
                return BaseParser.parse_datetime
            elif 6 <= _length <= 10:
                if ':' in _string:
                    return BaseParser.parse_time
                else:
                    return BaseParser.parse_date
            elif _length < 6:
                return BaseParser.parse_time
            else:
                return BaseParser.parse_special
        else:
            return BaseParser.__parse_not_only_str

    @staticmethod
    def __parse_not_only_str(string):
        baseParser_functions_to_try = [BaseParser.from_str, BaseParser.parse_diff]
        raised_exception = None
        for baseParser_function in baseParser_functions_to_try:
            try:
                return baseParser_function(string)
            except Exception as e:
                raised_exception = e

        if raised_exception:
            raise raised_exception

    @staticmethod
    def _datetime_parser(value):
        return value

    @staticmethod
    def _timestamp_parser(value):
        return datetime.datetime.fromtimestamp(value)

    @staticmethod
    def _special_parser(value):
        return value

    def _main_parser(func):
        def wrapper(cls, *args, **kwargs):
            value = args[0]
            if isinstance(value, str_tuple):
                method = BaseParser._str_parser(value)
            elif isinstance(value, (datetime.date, datetime.time, datetime.datetime)):
                method = BaseParser._datetime_parser
            elif isinstance(value, (int, float)):
                method = BaseParser._timestamp_parser
            else:
                method = BaseParser._special_parser

            if hasattr(method, '__call__'):
                return method(value)
            else:
                raise UnexpectedTypeError(
                    'can not generate method for {value} type:{type}'.format(value=value, type=type(value)))

        return wrapper

    @classmethod
    @_main_parser
    def main(cls, value):
        """parse all type value"""

    @staticmethod
    def parse_datetime(string, formation=None):
        if formation:
            _stamp = datetime.datetime.strptime(string, formation)
        elif len(string) >= 18:
            _stamp = datetime.datetime.strptime(string, '%Y-%m-%d %H:%M:%S')
        elif len(string) < 18:
            if '-' in string:
                _stamp = datetime.datetime.strptime(string, '%y-%m-%d %H:%M:%S')
            else:
                try:
                    _stamp = datetime.datetime.strptime(string, '%Y%m%d %H:%M:%S')
                except ValueError:
                    _stamp = datetime.datetime.strptime(string, '%y%m%d %H:%M:%S')
        else:
            raise CanNotFormatError('Need %Y-%m-%d %H:%M:%S or %y-%m-%d %H:%M:%S')
        return _stamp

    @staticmethod
    def parse_date(string, formation=None):
        """
        string to date stamp
        :param string: date string
        :param formation: format string
        :return: datetime.date
        """
        if formation:
            _stamp = datetime.datetime.strptime(string, formation).date()
            return _stamp

        _string = string.replace('.', '-').replace('/', '-')
        if '-' in _string:
            if len(_string.split('-')[0]) > 3 or len(_string.split('-')[2]) > 3:
                try:
                    _stamp = datetime.datetime.strptime(_string, '%Y-%m-%d').date()
                except ValueError:
                    try:
                        _stamp = datetime.datetime.strptime(_string, '%m-%d-%Y').date()
                    except ValueError:
                        _stamp = datetime.datetime.strptime(_string, '%d-%m-%Y').date()
            else:
                try:
                    _stamp = datetime.datetime.strptime(_string, '%y-%m-%d').date()
                except ValueError:
                    try:
                        _stamp = datetime.datetime.strptime(_string, '%m-%d-%y').date()
                    except ValueError:
                        _stamp = datetime.datetime.strptime(_string, '%d-%m-%y').date()
        else:
            if len(_string) > 6:
                try:
                    _stamp = datetime.datetime.strptime(_string, '%Y%m%d').date()
                except ValueError:
                    _stamp = datetime.datetime.strptime(_string, '%m%d%Y').date()
            elif len(_string) <= 6:
                try:
                    _stamp = datetime.datetime.strptime(_string, '%y%m%d').date()
                except ValueError:
                    _stamp = datetime.datetime.strptime(_string, '%m%d%y').date()
            else:
                raise CanNotFormatError
        return _stamp

    @staticmethod
    def parse_time(string):
        pass

    @staticmethod
    def parse_special(string):
        pass

    @staticmethod
    def parse_diff(base_str):
        """
        parse string to regular timedelta
        :param base_str: str
        :return: dict
        """
        temp_dict = {'years': 0,
                     'months': 0,
                     'weeks': 0,
                     'days': 0,
                     'hours': 0,
                     'minutes': 0,
                     'seconds': 0}

        _pure_str = re.findall("[a-zA-Z]+", base_str)
        pure_num = [int(_) for _ in re.findall(r'\d+', base_str)]
        pure_str = [filter_unit(_) for _ in _pure_str]
        result_dict = dict(chain(temp_dict.items(), dict(zip(pure_str, pure_num)).items()))
        if result_dict['months'] >= 12:
            advance = result_dict['months'] // 12
            remain = result_dict['months'] % 12
            result_dict['years'] += advance
            result_dict['months'] = remain
        if result_dict['weeks']:
            result_dict['days'] += result_dict['weeks'] * 7
        return result_dict

    @staticmethod
    def from_str(date):
        """
            Given a date in the format: Jan,21st.2015
            will return a datetime of it.
        """
        month = date[:3][0] + date[:3][-2:].lower()
        if month not in NAMED_MONTHS:
            raise CanNotFormatError('Month not recognized')

        date = date.replace(',', '').replace(' ', '').replace('.', '')
        try:
            day_unit = [x for x in ['st', 'rd', 'nd', 'th'] if x in date][0]
            day = int(re.search(r'\d+', date.split(day_unit)[0]).group())
            year = int(re.search(r'\d+', date.split(day_unit)[1]).group())
            numeric_month = NAMED_MONTHS[month]
            return datetime.date(int(year), numeric_month, day)
        except:
            raise CanNotFormatError('Not well formatted. Expecting something like May,21st.2015')


if __name__ == "__main__":
    BaseParser.main(_current)





class PyTimeException(Exception):
    """ A base class for exceptions used by pytime. """


class UnexpectedTypeError(PyTimeException):
    """unexpected type appears"""


class CanNotFormatError(PyTimeException):
    """parameter too odd"""



#!/usr/bin/env python
# encoding: utf-8

from setuptools import setup, find_packages
from codecs import open
from os import path

import pytime

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pytime',
    version=pytime.__version__,
    description='An easy-use module to operate datetime by string',
    long_description=long_description,
    url='https://github.com/shinux/PyTime',
    author='Sinux',
    author_email='nsinux@gmail.com',
    license='MIT',
    keywords='datetime time datetime timeparser',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    package_data={'': ['README.md']},
    tests_require=['coverage'],
    extras_require={
        'coveralls': ['coveralls']
    },
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
)
